package de.ibh.RandomArt;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

public class RAUtilities {
	public static double[] average(double[] c1, double[] c2, double w) {
		return new double[] {c1[0]*w + (1-w)*c2[0], c1[1]*w + (1-w)*c2[1], c1[2]*w + (1-w)*c2[2]};
	}
	
	public static double well(double x) {
		return 1 - 2 / ((1 + x*x) * (1 + x*x) * (1 + x*x) * (1 + x*x) * (1 + x*x) * (1 + x*x) * (1 + x*x) * (1 + x*x));
	}
	
	public static double tent(double x) {
		return 1 - 2 * Math.abs(x);
	}
	
	public static List<AbstractRAOperator> getOperators(final RAPRNG prng) {
		List<AbstractRAOperator> l = new LinkedList<AbstractRAOperator>();

		l.add(new RAOVarX(prng));
		l.add(new RAOVarY(prng));
		l.add(new RAOConstant(prng));
		l.add(new RAOSum(prng));
		l.add(new RAOProduct(prng));
		l.add(new RAOSin(prng));
		l.add(new RAOTent(prng));
		l.add(new RAOWell(prng));
		l.add(new RAOLevel(prng));
		l.add(new RAOMix(prng));

		return l;
	}
	
	public static List<AbstractRAOperator> filterOperatorsByArity(final List<AbstractRAOperator> ops, final Integer minArity, final Integer maxArity) {
		List<AbstractRAOperator> l = new LinkedList<AbstractRAOperator>();

		for(Iterator<AbstractRAOperator> i = ops.iterator(); i.hasNext(); ) {
			AbstractRAOperator op = i.next();
			
			if((minArity == null || op.getArity() >= minArity) &&
			   (maxArity == null || op.getArity() <= maxArity)) {
				l.add(op);
			}
		}

		return l;
	}
	
	public static void checkRGB(double rgb[]) {
		if(rgb.length != 3)
			throw new IllegalArgumentException("rgb.length(" + rgb.length + ") == 3");

		for(int i = 0; i < 3; i++) {
			if(rgb[i] == Double.NaN)
				throw new IllegalArgumentException("-1 <= rgb[" + i + "](NaN) <= 1");

			if(rgb[i] < -1 || rgb[i] > 1)
				throw new IllegalArgumentException("-1 <= rgb[" + i + "](" + rgb[i] + ") <= 1");
		}
	}
}
