package de.ibh.StrongRoom.Client.Config;

import java.io.IOException;
import java.net.UnknownHostException;
import java.util.InvalidPropertiesFormatException;
import java.util.NoSuchElementException;

import org.xbill.DNS.Lookup;
import org.xbill.DNS.SimpleResolver;
import org.xbill.DNS.TextParseException;
import org.xbill.DNS.Type;

import de.ibh.StrongRoom.Crypto.CryptoBackend.ECryptoPGP_KeyType;

public class AutoConf extends AbstractConf {
    static private AutoConf autoconf = null;
    String acPath;

    static public AutoConf getInstance() throws InvalidPropertiesFormatException, IOException {
        if(autoconf != null)
        	return autoconf;
        
        return autoconf = new AutoConf();
    }

    private AutoConf() throws InvalidPropertiesFormatException, IOException {
		super();
	}
    
    @Override
    public String getFileName() {
		try {
			Lookup l = new Lookup("_AutoConf._StrongRoom", Type.TXT);
			l.setResolver(new SimpleResolver());
			l.run();
	
			if(l.getResult() != Lookup.SUCCESSFUL)
				throw new NoSuchElementException();
			
			if(l.getAnswers().length < 1)
				throw new NoSuchElementException();
			
			acPath = l.getAnswers()[0].rdataToString().replaceAll("\"", "");
		} catch (TextParseException e) {
			throw new NoSuchElementException();
		} catch (UnknownHostException e) {
			throw new NoSuchElementException();
		}

		return acPath;
    }
    
	@Override
	public String getConfName() {
		return "auto";
	}

	
    public String getDBURL(final String jdbcCName) {
    	String p = prop.getProperty(BaseConf.SRP_CONFIG_DB_URL, "");

    	final int s = p.indexOf(jdbcCName) + jdbcCName.length() + 1;

    	if(s < jdbcCName.length() + 1)
    		return "";
    	
    	final int e = p.indexOf("\n", s);
    	
    	if(e > 0)
    		return p.substring(s, e);
    	else
    		return p.substring(s);
    }
	
    public String getMasKeyFPR() {
        return prop.getProperty(BaseConf.SRP_CONFIG_MASKEY_FPR, "");
    }

    public String getMasKeyData() {
        return prop.getProperty(BaseConf.SRP_CONFIG_MASKEY_DATA, "");
    }
    
	
	@Override
	public ECryptoPGP_KeyType getSignKeyType() {
		return null;
	}

	@Override
	public String getSignString() {
		return null;
	}

}
