/**
 * 
 */
package de.ibh.StrongRoom.Client.Config;

import java.io.File;
import java.io.IOException;
import java.util.Enumeration;
import java.util.InvalidPropertiesFormatException;
import java.util.LinkedList;
import java.util.List;

import de.ibh.StrongRoom.Crypto.CryptoBackend.ECryptoPGP_KeyType;

public class BaseConf extends AbstractConf {
	public static final String SRP_CONFIG_DB_JDBC = "StrongRoom.Config.DB.JDBC";
    public static final String SRP_CONFIG_DB_URL = "StrongRoom.Config.DB.URL";
    public static final String SRP_CONFIG_DB_CREDMETHOD = "StrongRoom.Config.DB.CredentialMethod";
    public static final String SRP_CONFIG_DB_USERNAME = "StrongRoom.Config.DB.Username";
    public static final String SRP_CONFIG_DB_PASSWORD = "StrongRoom.Config.DB.Password";

    public static final String SRP_CONFIG_SECKEY_FPR = "StrongRoom.Config.PGP.SecretKey.FPR";
    public static final String SRP_CONFIG_SECKEY_METHOD = "StrongRoom.Config.PGP.SecretKey.Method";
    public static final String SRP_CONFIG_SECKEY_GPGBIN = "StrongRoom.Config.PGP.SecretKey.GnuPG";
    public static final String SRP_CONFIG_SECKEY_DATA = "StrongRoom.Config.PGP.SecretKey.Data";

    public static final String SRP_CONFIG_MASKEY_FPR = "StrongRoom.Config.PGP.MasterKey.FPR";
    public static final String SRP_CONFIG_MASKEY_DATA = "StrongRoom.Config.PGP.MasterKey.Data";

    public static final String SRP_CONFIG_SIGNATURE = "StrongRoom.Config.Signature";

    public enum EDBCredentialMethod {
        ENCRYPTED, NONE, ASK;
    }
    
    public enum ESecretKeySource {
        PLAIN, GNUPG;
    }
    
    static private BaseConf baseconf = null;

    static public BaseConf getInstance() throws InvalidPropertiesFormatException, IOException {
        if(baseconf != null)
        	return baseconf;
        
        return baseconf = new BaseConf();
    }

    static public BaseConf getInstanceSilent() {
        if(baseconf != null)
        	return baseconf;
        
        return baseconf = new BaseConf(true);
    }

    private BaseConf() throws InvalidPropertiesFormatException, IOException {
		super();
	}

    private BaseConf(boolean silent) {
		super(silent);
	}

    public static void dispose() {
    	baseconf = null;
    }
    
	@Override
	public String getConfName() {
		return "base";
	}

	public static boolean confFileExists() {
		File f = new File(getDirName() + "base.xml");
		
		return f.exists();
	}
    
    public EDBCredentialMethod getDBCredentialMethod() {
        try {
        	return EDBCredentialMethod.valueOf(
        			prop.getProperty(SRP_CONFIG_DB_CREDMETHOD, EDBCredentialMethod.ASK.name())
        	);
        }
        catch(Exception e) {
            if (System.getProperty("os.name").contains("Windows"))
            	return EDBCredentialMethod.NONE;

           	return EDBCredentialMethod.ASK;
        }
    }
    
    public void setDBCredentialMethod(final EDBCredentialMethod cm) {
        prop.setProperty(SRP_CONFIG_DB_CREDMETHOD, cm.name()); 
    }
    
    public String getDBJDBC() {
        return prop.getProperty(SRP_CONFIG_DB_JDBC, "net.sourceforge.jtds.jdbc.Driver"); 
    }
    
    public void setDBJDBC(final String jdbc) {
        prop.setProperty(SRP_CONFIG_DB_JDBC, jdbc); 
    }
    
    public String getDBURL() {
    	if(getDBCredentialMethod() == EDBCredentialMethod.NONE)
            return prop.getProperty(SRP_CONFIG_DB_URL, "jdbc:odbc:StrongRoom"); 
    		
        return prop.getProperty(SRP_CONFIG_DB_URL, "jdbc:jtds:sqlserver://strongroom-db-host/StrongRoom;progName=StrongRoom;user=%DB_USERNAME%;password=%DB_PASSWORD%"); 
    }
    
    public void setDBURL(final String url) {
        prop.setProperty(SRP_CONFIG_DB_URL, url); 
    }
    
    public String getDBUsername() {
        return prop.getProperty(SRP_CONFIG_DB_USERNAME, ""); 
    }

    public void setDBUsername(final String username) {
        prop.setProperty(SRP_CONFIG_DB_USERNAME, username == null ? "" : username); 
    }
    
    public String getDBPassword() {
        return prop.getProperty(SRP_CONFIG_DB_PASSWORD, ""); 
    }    
    
    public void setDBPassword(final String password) {
        prop.setProperty(SRP_CONFIG_DB_PASSWORD, password == null ? "" : password); 
    }    

    
    
    public String getSecretKeyFPR() {
        return prop.getProperty(SRP_CONFIG_SECKEY_FPR, "");
    }
    
    public void setSecretKeyFPR(final String fpr) {
        prop.setProperty(SRP_CONFIG_SECKEY_FPR, fpr);
    }
    
    public ESecretKeySource getSecretKeyKeyMethod() {
    	try {
    		return ESecretKeySource.valueOf(
    				prop.getProperty(SRP_CONFIG_SECKEY_METHOD, ESecretKeySource.GNUPG.name())
    		);
    	}
    	catch(Exception e) {
            return ESecretKeySource.GNUPG;
    	}
    }
    
    public void setSecretKeyKeyMethod(final ESecretKeySource km) {
    	prop.setProperty(SRP_CONFIG_SECKEY_METHOD, km.name());
    }
    
    public String getSecretKeyGPGBin() {
        return prop.getProperty(SRP_CONFIG_SECKEY_GPGBIN, "gpg");
    }
    
    public void setSecretKeyGPGBin(final String gpg) {
        prop.setProperty(SRP_CONFIG_SECKEY_GPGBIN, gpg);
    }
    
    public String getSecretKeyData() {
        return prop.getProperty(SRP_CONFIG_SECKEY_DATA, "");
    }
    
    public void setSecretKeyData(final String data) {
        prop.setProperty(SRP_CONFIG_SECKEY_DATA, data);
    }

    
    
    public String getMasKeyFPR() {
        return prop.getProperty(SRP_CONFIG_MASKEY_FPR, "");
    }
    
    public void setMasKeyFPR(final String fpr) {
        prop.setProperty(SRP_CONFIG_MASKEY_FPR, fpr);
    }

    public String getMasKeyData() {
        return prop.getProperty(SRP_CONFIG_MASKEY_DATA, "");
    }
    
    public void setMasKeyData(final String data) {
        prop.setProperty(SRP_CONFIG_MASKEY_DATA, data);
    }

    
    
    public String getSignature() {
        return prop.getProperty(SRP_CONFIG_SIGNATURE, "");
    }
    
    public void setSignature(final String sig) {
        prop.setProperty(SRP_CONFIG_SIGNATURE, sig);
    }


    
	@Override
	public ECryptoPGP_KeyType getSignKeyType() {
		return ECryptoPGP_KeyType.USER_KEY;
	}

	@Override
	public String getSignString() {
		List<String> l = new LinkedList<String>();
		
		for(Enumeration<Object> keys = prop.keys(); keys.hasMoreElements(); ) {
			String key = keys.nextElement().toString();
			
			if(!key.equals(SRP_CONFIG_SIGNATURE))
				l.add(key + "=" + prop.getProperty(key));
		}
		
		return buildSignString(l);
	}
}