package de.ibh.StrongRoom.Client;

import java.sql.SQLException;

import de.ibh.StrongRoom.Client.Config.BaseConf;
import de.ibh.StrongRoom.Client.Config.SigningConf;
import de.ibh.StrongRoom.Crypto.IPassphraseRequest;
import de.ibh.StrongRoom.Crypto.CryptoBackend.ECryptoPGP_KeyType;
import de.ibh.StrongRoom.DB.DBConn;
import de.ibh.StrongRoom.Exception.SRDBConnException;
import de.ibh.StrongRoom.Exception.SRGenericCryptoEngineException;
import de.ibh.StrongRoom.Exception.SRGenericCryptoModelException;
import de.ibh.StrongRoom.Log.ErrorMessage;
import de.ibh.StrongRoom.Log.ErrorMessage.EMessageTypes;

public class DB {
    private static     DBConn dbconn     = null;
    private static     int    retryCount = 3; 
    
    public static DBConn getDBConn(Crypto crypto, BaseConf baseconf, SigningConf signingconf, UserInput input, IPassphraseRequest ppr, ISigningKeyVerificationRequest skvr, Object userData, ErrorMessage errmsg) throws SRGenericCryptoModelException, SRGenericCryptoEngineException {
        String db_username   = null;
        String db_password   = null;
        String db_dsn        = null;
        
        if(dbconn != null && dbconn.isConnected())
        	dbconn.disconnect();

        if((db_dsn = baseconf.getDBURL()) == null) {
            errmsg.printErrorMessage("Database DSN not definied in the configuration file.", EMessageTypes.TYPE_ERROR);
            return null;  
        }
        
        switch(baseconf.getDBCredentialMethod()) {
        case ENCRYPTED:
            if(ppr == null) return null;

            for(int i=0; i<retryCount; i++) {
                String cDb_password = baseconf.getDBPassword();
                if(cDb_password == null || cDb_password.equals("")) {
                    errmsg.printErrorMessage("No encrypted password found in configuration file!", EMessageTypes.TYPE_ERROR);
                    return null;
                }
                
                db_username = baseconf.getDBUsername();

                try {
					db_password = Misc.b2s(crypto.decrypt(ECryptoPGP_KeyType.USER_KEY, cDb_password.getBytes(), ppr, null, userData));
	               	i = retryCount;
				} catch (SRGenericCryptoEngineException e) {
                	errmsg.printErrorMessage("Decryption of the database password failed: " + e.getLocalizedMessage(), EMessageTypes.TYPE_WARN);
				}
            }
            break;
           
        case NONE:
            db_username = "";
            db_password = "";
            
            break;
          
        case ASK:
        default:
            if(input == null)
            	return null;

            db_username = input.userInputText("DB Logon", "Please enter the DB username:");
            db_password = input.userInputPassword("DB Logon", "Please enter the DB password:");
        }
        
        if(db_dsn == null || db_username == null || db_password == null)
        	return null;
        
        DBConn dbconn;
		try {
			String jURL = baseconf.getDBURL().replaceAll("%DB_USERNAME%", db_username).replaceAll("%DB_PASSWORD%", db_password);
			
			dbconn = DBConn.getInstance(baseconf.getDBJDBC(), jURL, errmsg);

			jURL = null;
		} catch (InstantiationException e) {
			throw new SRDBConnException(e);
		} catch (IllegalAccessException e) {
			throw new SRDBConnException(e);
		} catch (ClassNotFoundException e) {
			throw new SRDBConnException(e);
		} catch (SQLException e) {
			throw new SRDBConnException(e);
		}
		finally {
			/* forget confidential data */
			db_password = null;

			System.gc();
		}

        crypto.verifySigningKey(signingconf, dbconn.getSigningKey(), ppr, skvr, userData);
        
        return dbconn;
    }
}