package de.ibh.StrongRoom.Client.Lucene;

import de.ibh.StrongRoom.Client.Crypto;
import de.ibh.StrongRoom.Client.Misc;
import de.ibh.StrongRoom.Client.SWT.PassphraseDialog;
import de.ibh.StrongRoom.Crypto.IRetrievePublicKeyRingHelper;
import de.ibh.StrongRoom.Crypto.IRetrieveSecretKeyRingHelper;
import de.ibh.StrongRoom.Crypto.CryptoBackend.ECryptoPGP_KeyType;
import de.ibh.StrongRoom.DB.Role;
import de.ibh.StrongRoom.Exception.SRGenericCryptoEngineException;
import de.ibh.StrongRoom.Exception.SRPublicKeyNotFoundException;
import de.ibh.StrongRoom.Exception.SRSecretKeyNotFoundException;

public class SRCrypto {
	Crypto crypto;
	Role role;
	IRetrievePublicKeyRingHelper pubHelper;
	IRetrieveSecretKeyRingHelper secHelper;
	Object userData;
	String roleKey;
	
	public SRCrypto(Crypto crypto, Role role, IRetrievePublicKeyRingHelper pubHelper, IRetrieveSecretKeyRingHelper secHelper, Object userData) {
		this.crypto = crypto;
		this.role = role;
		this.pubHelper = pubHelper;
		this.secHelper = secHelper;
		this.userData = userData;
		this.roleKey = null;
	}
	
    public byte[] encryptOutput(byte[] data) throws SRPublicKeyNotFoundException, SRGenericCryptoEngineException {
    	ECryptoPGP_KeyType ka[] = new ECryptoPGP_KeyType[] {ECryptoPGP_KeyType.MASTER_KEY, ECryptoPGP_KeyType.ROLE_KEY};

    	return crypto.encrypt(ka, data, pubHelper, userData);
    }
	
    public byte[] decryptInput(byte[] data) throws SRSecretKeyNotFoundException, SRGenericCryptoEngineException {
		if(roleKey == null)
			roleKey = Misc.b2s(crypto.decrypt(ECryptoPGP_KeyType.USER_KEY, role.getRoleKey().getEncryptedData().getBytes(), PassphraseDialog.get(), null, userData));

		return crypto.decrypt(roleKey, data, userData);
    }
}
