package de.ibh.StrongRoom.Client.Lucene;

import java.io.IOException;

import org.apache.lucene.store.Directory;
import org.apache.lucene.store.IndexInput;
import org.apache.lucene.store.IndexOutput;

import de.ibh.StrongRoom.Client.Crypto;
import de.ibh.StrongRoom.Crypto.IRetrievePublicKeyRingHelper;
import de.ibh.StrongRoom.Crypto.IRetrieveSecretKeyRingHelper;
import de.ibh.StrongRoom.DB.DBConn;
import de.ibh.StrongRoom.DB.LuceneStore;
import de.ibh.StrongRoom.DB.Role;

public class SRDirectory extends Directory {
	DBConn db;
	Role role;
	SRCrypto src;
	
	public SRDirectory(final DBConn db, final Crypto crypto, final Role role, IRetrievePublicKeyRingHelper pubHelper, IRetrieveSecretKeyRingHelper secHelper, Object userData) throws IOException {
		super();

		setLockFactory(new SRNoLockFactory());
		
		this.db = db;
		this.role = role;

		src = new SRCrypto(crypto, role, pubHelper, secHelper, userData);
	}

	
	@Override
	public void close() throws IOException {
		return;
	}

	@Override
	public IndexOutput createOutput(String arg0) throws IOException {
		return new SRIndexOutput(new LuceneStore(role, arg0), db, src);
	}

	
	@Override
	public void deleteFile(String arg0) throws IOException {
		db.remLuceneStore(new LuceneStore(role, arg0));
	}

	@Override
	public boolean fileExists(String arg0) throws IOException {
		return db.existsLuceneStore(new LuceneStore(role, arg0));
	}

	@Override
	public long fileLength(String arg0) throws IOException {
		return db.sizeLuceneStore(new LuceneStore(role, arg0));
	}

	@Override
	public long fileModified(String arg0) throws IOException {
		return db.modifiedLuceneStore(new LuceneStore(role, arg0));
	}

	@Override
	public String[] listAll() throws IOException {
		return db.listLuceneStores(role);
	}

	@Override
	public IndexInput openInput(String arg0) throws IOException {
		try {
			return new SRIndexInput(new LuceneStore(role, arg0), db, src);
		} catch (Exception e) {
			throw new IOException(e);
		}
	}

	@Override
	public void touchFile(String arg0) throws IOException {
		db.touchLuceneStore(new LuceneStore(role, arg0));
	}

	public void clearIndex() {
		db.clearLuceneStore(role);
	}
}
