package de.ibh.StrongRoom.Client.Lucene;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import org.apache.lucene.store.IndexOutput;

import de.ibh.StrongRoom.DB.DBConn;
import de.ibh.StrongRoom.DB.LuceneStore;

public class SRIndexOutput extends IndexOutput {
	ByteArrayOutputStream os; 
	int pos;
	LuceneStore file;
	DBConn db;
	SRCrypto src;
	
	public SRIndexOutput(final LuceneStore file, final DBConn db, final SRCrypto src) {
		os = new ByteArrayOutputStream();
		pos = 0;
		this.file = file;
		this.db = db;
		this.src = src;
	}
	
	@Override
	public void close() throws IOException {
		try {
			System.err.println(file.getName() + ".close("+os.size()+")");
			db.setLuceneStoreContent(file, src.encryptOutput(os.toByteArray()), os.size());
		} catch (Exception e) {
			throw new IOException(e);
		}
	}

	@Override
	public void flush() throws IOException {
		try {
			db.setLuceneStoreContent(file, src.encryptOutput(os.toByteArray()), os.size());
		} catch (Exception e) {
			throw new IOException(e);
		}
	}

	@Override
	public long getFilePointer() {
		System.err.println(file.getName() + ".getFilePointer() = " + pos);
		return pos;
	}

	@Override
	public long length() throws IOException {
		System.err.println(file.getName() + ".length() = " + os.size());
		return os.size();
	}

	@Override
	public void seek(long p) throws IOException {
		System.err.println(file.getName() + ".seek(" + p + ")");
		pos = (int) p;
	}

	@Override
	public void writeByte(byte b) throws IOException {
		if(pos < os.size()) {
			byte[] ba = os.toByteArray();

			os = new ByteArrayOutputStream();
			os.write(ba, 0, pos);
			os.write(b);
			os.write(ba, pos + 1, ba.length - pos - 1);
			System.err.println(">*<");
		}
		else
			os.write(b);

		pos++;

		System.err.println(file.getName() + ".writeByte(" + Integer.toHexString(b) + "): "+ pos);
	}

	@Override
	public void writeBytes(byte[] b, int off, int l) throws IOException {
		if(pos < os.size()) {
			byte[] ba = os.toByteArray();

			os = new ByteArrayOutputStream();
			os.write(ba, 0, pos);
			os.write(b, off, l);
			os.write(ba, pos + l, ba.length - pos - l);
			System.err.println(">*<");
		}
		else
			os.write(b, off, l);

		pos += l;

		System.err.println(file.getName() + ".writeBytes("+off+","+l+"): "+ pos);
	}
}
