/**
 * 
 */
package de.ibh.StrongRoom.Client.SWT;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;

import de.ibh.StrongRoom.l10n.StrongRoomL10N;

/**
 * @author ellguth
 *
 */
public class MainMenu {
    private Menu m = null, mFile = null, mEdit = null, mView = null, mTools = null, mHelp = null;
    private static MainMenu mainmenu = null;
    private final StrongRoomL10N l10n = StrongRoomL10N.getInstance();     
    
    static public final int ITEM_FILE         =       1;
    static public final int ITEM_EDIT         =       2;
    static public final int ITEM_COPY         =       4;
    static public final int ITEM_PASTE        =       8;
    static public final int ITEM_CUT          =      16;
    static public final int ITEM_EXIT         =      32;
    static public final int ITEM_HELP         =      64;
    static public final int ITEM_SAVE         =     128;
    static public final int ITEM_CLOSE        =     256;
    static public final int ITEM_OPEN         =     512;
    static public final int ITEM_CONNECT      =    1024;
    static public final int ITEM_DISCONNECT   =    2048;
    static public final int ITEM_ABOUT        =    4096;
    static public final int ITEM_PREFERENCES  =    8192;
    static public final int ITEM_IMPORT       =   16384;
    static public final int ITEM_SINDEX       =   32768;
    static public final int ITEM_FIND         =   65536;
    static public final int ITEM_TOOLS        =   71072;
    static public final int ITEM_VIEW         =  131172;
    static public final int ITEM_SHOWHIDDEN   =  262344;
    static public final int ITEM_COLLAPSELIST =  524688;
    static public final int ITEM_CLOSETAB     = 1049376;
    static public final int ITEM_HIGHLIGHTALL = 2098752;
    
    
    private Map<Integer,MenuItem> _items = new HashMap<Integer, MenuItem>();    
    
    
    static public MainMenu getInstance(Shell parent) {
        if(mainmenu != null)
            return mainmenu;
        
        mainmenu = new MainMenu(parent);
        
        return mainmenu;        
    }
    
    /**
     * @param parent
     */
    public MainMenu(Shell parent) {
        m = new Menu(parent, SWT.BAR);
        
        addMenuItems(parent);
        parent.setMenuBar(m);
    }
    
    
    public int getItemMask() {
        int mask = 0;
        
        Iterator<Map.Entry<Integer, MenuItem>> items = _items.entrySet().iterator();
        
        while (items.hasNext()) {
            Map.Entry<Integer, MenuItem> item = items.next();
            if(((MenuItem) item.getValue()).getEnabled() == true)
                mask|=item.getKey();
        }
        
        return mask;
    }
    
    
    public boolean getItemSelection(int item) {
        MenuItem _item = _items.get(item);
        if(_item == null) return false;
        
        return _item.getSelection();
    }
    
    
    public void setItemSelection(int item, boolean selection) {
        MenuItem _item = _items.get(item);
        if(_item == null) return;
        
        _item.setSelection(selection);
    }
    
    
    public void addItemListener(int item, Listener listener) {
        MenuItem _item = _items.get(item);
        if(_item == null) return;
        
        _item.setData(item);
        _item.addListener(SWT.Selection, listener);
    }
    
    
    public void removeItemListener(int item) {
        MenuItem _item = _items.get(item);
        if(_item == null) return;
        
        for(Listener listener : _item.getListeners(SWT.Selection))
            _item.removeListener(SWT.Selection, listener);
    }    
    
    
    public void disableItems(int disitems) {
        Iterator<Map.Entry<Integer, MenuItem>> items = _items.entrySet().iterator();
        
        while (items.hasNext()) {
            Map.Entry<Integer, MenuItem> item = items.next();
            if(((Integer) item.getKey() & disitems) == (Integer) item.getKey()) {
                ((MenuItem) item.getValue()).setEnabled(false);
            }
            
        }        
    }
    

    public void enableItems(int enaitems) {
        Iterator<Map.Entry<Integer, MenuItem>> items = _items.entrySet().iterator();
        
        while (items.hasNext()) {
            Map.Entry<Integer, MenuItem> item = items.next();
            if(((Integer) item.getKey() & enaitems) == (Integer) item.getKey()) {
                ((MenuItem) item.getValue()).setEnabled(true);
            }
            
        }        
    }
    
    
    public HashMap<Integer, Listener []> getItemListeners() {
        HashMap<Integer, Listener []> hashmap = new HashMap<Integer, Listener[]>();
        
        for(Iterator<Integer> it = _items.keySet().iterator(); it.hasNext();) {
            Integer key = it.next();
            hashmap.put(key, _items.get(key).getListeners(SWT.Selection));
        }
        
        return hashmap;
    }
    
    
    public void setItemListeners(HashMap<Integer, Listener []> hashmap) {
        for(Iterator<Integer> it = hashmap.keySet().iterator(); it.hasNext();) {
            Integer key = it.next();
            removeItemListener(key);
            for(Listener listener : hashmap.get(key))
                addItemListener(key, listener);
        }
    }
    
    
    public void removeAllItemListeners() {
        for(Iterator<Integer> it = _items.keySet().iterator(); it.hasNext();)
            removeItemListener(it.next());
    }    
    
    
    public void disableAllItems() {
        disableItems(0xFFFF);
    }
            
    public void enableAllItems() {
        enableItems(0xFFFF);
    }    
    
    
    private void addMenuFile(Shell parent) {
        MenuItem _item = new MenuItem(mFile, SWT.PUSH);
        _item.setText(l10n._("&Connect"));
        _items.put(ITEM_CONNECT, _item);
        
        _item = new MenuItem(mFile, SWT.PUSH);
        _item.setText(l10n._("&Disconnect"));
        _items.put(ITEM_DISCONNECT, _item);
        
	    _item = new MenuItem(mFile, SWT.PUSH);
	    _item.setText(l10n._("&Preferences"));
	    _items.put(ITEM_PREFERENCES, _item);

	    new MenuItem(mFile, SWT.SEPARATOR);

        _item = new MenuItem(mFile, SWT.PUSH);
        _item.setText(l10n._("&Save") + "\tCtrl+S");
        _item.setAccelerator(SWT.CTRL + 'S');
        _items.put(ITEM_SAVE, _item);
        
        _item = new MenuItem(mFile, SWT.PUSH);
        _item.setText(l10n._("&Close") + "\tESC");
        _item.setAccelerator(SWT.ESC);
        _items.put(ITEM_CLOSE, _item);
        
        _item = new MenuItem(mFile, SWT.PUSH);
        _item.setText(l10n._("Close Tab") + "\tCTRL+W");
        _item.setAccelerator(SWT.CTRL + 'W');
        _items.put(ITEM_CLOSETAB, _item);        

        new MenuItem(mFile, SWT.SEPARATOR);

        _item = new MenuItem(mFile, SWT.PUSH);
        _item.setText(l10n._("&Quit"));
        _items.put(ITEM_EXIT, _item);
    }
    
    
    private void addMenuEdit(Shell parent) {
        MenuItem _item = new MenuItem(mEdit, SWT.PUSH);
        _item.setText(l10n._("Cut") + "\tCtrl+X");
        _items.put(ITEM_CUT, _item);
        
        _item = new MenuItem(mEdit, SWT.PUSH);
        _item.setText(l10n._("Copy") + "\tCtrl+C");
        _items.put(ITEM_COPY, _item);
        
        _item = new MenuItem(mEdit, SWT.PUSH);
        _item.setText(l10n._("Paste") + "\tCtrl+V");
        _items.put(ITEM_PASTE, _item);
        
        new MenuItem(mEdit, SWT.SEPARATOR);
        
        _item = new MenuItem(mEdit, SWT.PUSH);
        _item.setText(l10n._("&Find")+ "\tCtrl+F");
        _item.setAccelerator(SWT.CTRL + 'F');
        _items.put(ITEM_FIND, _item);        
    }
    
    private void addMenuTools(Shell parent) {
	    MenuItem _item = new MenuItem(mTools, SWT.PUSH);
	    _item.setText(l10n._("&Import data"));
	    _items.put(ITEM_IMPORT, _item);        
	    
	    _item = new MenuItem(mTools, SWT.PUSH);
	    _item.setText(l10n._("&Build search index"));
	    _items.put(ITEM_SINDEX, _item);
    }
    
    private void addMenuView(Shell parent) {
        Menu _submenu = new Menu(parent, SWT.DROP_DOWN);

        MenuItem _item = new MenuItem(mView, SWT.CASCADE);
        _item.setText(l10n._("Quick Search"));
        _item.setMenu(_submenu);
        
        _item = new MenuItem(_submenu, SWT.CHECK);
        _item.setText(l10n._("Show hidden entries"));
        _items.put(ITEM_SHOWHIDDEN, _item);

        new MenuItem(mView, SWT.SEPARATOR);
        
        _item = new MenuItem(mView, SWT.CHECK);
        _item.setText(l10n._("Collapse Version history"));
        _items.put(ITEM_COLLAPSELIST, _item);
        
        _item = new MenuItem(mView, SWT.CHECK);
        _item.setText(l10n._("Highlight all (as default)"));
        _items.put(ITEM_HIGHLIGHTALL, _item);        
    }    
    
    private void addMenuHelp(Shell parent) {
        MenuItem _item = new MenuItem(mHelp, SWT.PUSH);
        _item.setText(l10n._("About"));
        _items.put(ITEM_ABOUT, _item);
    }
    
    public MenuItem getItem(int id) {
    	return _items.get(id);
    }
    
    private void addMenuItems(Shell parent) {
        mFile = new Menu(parent, SWT.DROP_DOWN);
        mEdit = new Menu(parent, SWT.DROP_DOWN);
        mView = new Menu(parent, SWT.DROP_DOWN);
        mTools = new Menu(parent, SWT.DROP_DOWN);        
        mHelp = new Menu(parent, SWT.DROP_DOWN);
        
        MenuItem _item = new MenuItem(m, SWT.CASCADE);
        _item.setText(l10n._("&File"));
        _item.setMenu(mFile);
        _items.put(ITEM_FILE, _item);
        
        _item = new MenuItem(m, SWT.CASCADE);
        _item.setText(l10n._("&Edit"));
        _item.setMenu(mEdit);
        _items.put(ITEM_EDIT, _item);
        
        _item = new MenuItem(m, SWT.CASCADE);
        _item.setText(l10n._("&View"));
        _item.setMenu(mView);
        _items.put(ITEM_VIEW, _item);        
        
        _item = new MenuItem(m, SWT.CASCADE);
        _item.setText(l10n._("&Tools"));
        _item.setMenu(mTools);
        _items.put(ITEM_TOOLS, _item);
        
        _item = new MenuItem(m, SWT.CASCADE);
        _item.setText(l10n._("&Help"));
        _item.setMenu(mHelp);
        _items.put(ITEM_HELP, _item);
        
        addMenuEdit(parent);
        addMenuFile(parent);
        addMenuView(parent);
        addMenuTools(parent);
        addMenuHelp(parent);
    }
}
