package de.ibh.StrongRoom.Client.SWT;

import java.io.IOException;

import org.apache.lucene.index.CorruptIndexException;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Shell;

import de.ibh.StrongRoom.Client.Crypto;
import de.ibh.StrongRoom.Client.Config.BaseConf;
import de.ibh.StrongRoom.Client.Lucene.ISRIndexUpdateProgress;
import de.ibh.StrongRoom.Client.Lucene.SRSearch;
import de.ibh.StrongRoom.Crypto.IRunnable;
import de.ibh.StrongRoom.DB.DBConn;
import de.ibh.StrongRoom.DB.Role;
import de.ibh.StrongRoom.Exception.SRGenericException;

public class SearchIdxBuilder {
	public SearchIdxBuilder(final Shell s, final DBConn dbconn, final BaseConf baseconf, final Crypto crypto, final Role role) throws SRGenericException, CorruptIndexException, IOException {
		final Shell shell = new Shell(s, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);

		final Label statusLabel = new Label(shell, SWT.NONE);
		statusLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
		
        final ProgressBar progressBar = new ProgressBar(shell, SWT.NULL); 
        progressBar.setMaximum(100);
        progressBar.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
        progressBar.setEnabled(false);

        
		final ISRIndexUpdateProgress up = new ISRIndexUpdateProgress() {
			@Override
			public void init(final String status, final int max) {
				shell.getDisplay().syncExec(new Runnable() {
					
					@Override
					public void run() {
						statusLabel.setText(status);
						statusLabel.update();

						progressBar.setMaximum(max);
						progressBar.setSelection(0);
						progressBar.update();
					}
				});
			}

			@Override
			public void progress(final int pos) {
				shell.getDisplay().syncExec(new Runnable() {
					
					@Override
					public void run() {
						progressBar.setSelection(pos);
						progressBar.update();
					}
				});
			}
		};
		
		shell.pack();
		shell.open();

		final IRunnable ira = new IRunnable() {

			@Override
			public void run(Runnable ra) {
				shell.getDisplay().syncExec(ra);
			}
		};
		
		final Thread thBuild = new Thread() {
			public void run() {
				try {
					SRSearch.buildIndex(up, role, dbconn, crypto, ira, shell);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		};
		thBuild.start();
		
		Display display = shell.getDisplay();
	    while (!shell.isDisposed()) {
	    	if (!display.readAndDispatch())
	    		display.sleep();
	    	
	    	if(!thBuild.isAlive())
	    		shell.dispose();
	    }
	}
}
