/**
 * 
 */
package de.ibh.StrongRoom.Client.SWT;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import de.ibh.RandomArt.RAImage;
import de.ibh.StrongRoom.Client.ISigningKeyVerificationRequest;
import de.ibh.StrongRoom.l10n.StrongRoomL10N;


/**
 * @author ellguth
 *
 */
public class SigningKeyVerificationDialog implements ISigningKeyVerificationRequest {
    private StrongRoomL10N l10n = StrongRoomL10N.getInstance();
	private static SigningKeyVerificationDialog self = null;
	boolean ret;

	public static SigningKeyVerificationDialog get() {
		if(self == null)
			self = new SigningKeyVerificationDialog();
		
		return self;
	}
	
	private SigningKeyVerificationDialog() {
		super();
	}

	@Override
	public boolean verifySigningKey(final long oldSerial, final String oldFPR,
			final long currentSerial, final String currentFPR, final Object userData) {
		Shell parent = (Shell) userData;
		
		final Shell shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);

		shell.setLayout(new GridLayout(2, false));
		
	    /* label */
	    Label label = new Label(shell, SWT.NONE);
	    if(oldSerial == 0) {
	    	label.setText(l10n._("There was no local signing key configuration found. Please verify that the current signing key is valid before confirming the current signing key!"));
	    }
	    else {
	    	label.setText(l10n._("The signing key has been changed. Please verify that the current signing key has been changed before confirming the current signing key!"));
	    }
	    label.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, true, false, 2, 1));

	    
	    if(oldSerial != 0) {
		    /* Previous Signing Key */
	        Group previousGroup = new Group(shell, SWT.BORDER);
	        previousGroup.setText(l10n._("Previous Signing Key"));
	        previousGroup.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
	        previousGroup.setLayout(new GridLayout(3, false));
		    
		    /*  Random Art  */
			Canvas canvasFPR = new Canvas(previousGroup, SWT.NONE);
			canvasFPR.addPaintListener(new PaintListener() {
				public void paintControl(PaintEvent e) {
					final Image img = RAImage.create(oldFPR, shell.getDisplay());
					e.gc.drawImage(img, 0, 0);
				}
			});
			GridData gd = new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 2);
			gd.heightHint = RAImage.getDefaultHeight();
			gd.minimumHeight = RAImage.getDefaultHeight();
			gd.widthHint = RAImage.getDefaultWidth();
			gd.minimumWidth = RAImage.getDefaultWidth();
			canvasFPR.setLayoutData(gd);
		    
		    /*  Serial  */
			label = new Label(previousGroup, SWT.NONE);
		    label.setText(l10n._("Serial:"));
		    label.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
	
			label = new Label(previousGroup, SWT.NONE | SWT.BOLD);
			label.setText("#" + oldSerial);
		    label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
	
		    
		    /*  FPR  */
			label = new Label(previousGroup, SWT.NONE);
		    label.setText("FPR:");
		    label.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));
	
			label = new Label(previousGroup, SWT.NONE | SWT.BOLD);
		    label.setText(oldFPR);
		    label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
	    }
	    

	    /* Current Signing Key */
	    
        final Group currentGroup = new Group(shell, SWT.NONE);
        currentGroup.setText(l10n._("Current Signing Key"));
        currentGroup.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
        currentGroup.setLayout(new GridLayout(3, false));
	    
	    /*  Random Art  */
		Canvas canvasFPR = new Canvas(currentGroup, SWT.NONE);
		canvasFPR.addPaintListener(new PaintListener() {
			public void paintControl(PaintEvent e) {
				final Image img = RAImage.create(currentFPR, shell.getDisplay());
				e.gc.drawImage(img, 0, 0);
			}
		});
		GridData gd = new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 2);
		gd.heightHint = RAImage.getDefaultHeight();
		gd.minimumHeight = RAImage.getDefaultHeight();
		gd.widthHint = RAImage.getDefaultWidth();
		gd.minimumWidth = RAImage.getDefaultWidth();
		canvasFPR.setLayoutData(gd);
	    
	    /*  Serial  */
		label = new Label(currentGroup, SWT.NONE);
	    label.setText(l10n._("Serial:"));
	    label.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));

		label = new Label(currentGroup, SWT.NONE | SWT.BOLD);
		label.setText("#" + currentSerial);
	    label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

	    
	    /*  FPR  */
		label = new Label(currentGroup, SWT.NONE);
	    label.setText("FPR:");
	    label.setLayoutData(new GridData(SWT.END, SWT.CENTER, false, false));

		label = new Label(currentGroup, SWT.NONE | SWT.BOLD);
	    label.setText(currentFPR);
	    label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

	    
	    /* Verified button */

	    final Button ok = new Button(shell, SWT.PUSH);
	    ok.setText(l10n._("Verified"));
	    ok.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, false));
	    ok.addSelectionListener(new SelectionAdapter() {
	    	public void widgetSelected(SelectionEvent event) {
	    		ret = true;
	    		shell.close();
	    	}
	    });
	    ok.setImage(shell.getDisplay().getSystemImage(SWT.OK));

	    
	    /* Cancel button */
	    Button cancel = new Button(shell, SWT.PUSH);
	    cancel.setImage(shell.getDisplay().getSystemImage(SWT.CANCEL));
	    cancel.setText(l10n._("Cancel"));
	    cancel.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, false));
	    cancel.addSelectionListener(new SelectionAdapter() {
	      public void widgetSelected(SelectionEvent event) {
	        shell.close();
	      }
	    });
	    shell.setDefaultButton(cancel);

	    ret = false;
	    
	    shell.pack();
	    shell.open();
	    Display display = parent.getDisplay();
	    while (!shell.isDisposed()) {
	      if (!display.readAndDispatch()) {
	        display.sleep();
	      }
	    }
	    
	    return ret;
	}
}
