//---------------------------------------------------------------------------
// Freely available from Media Art Online (http://www.media-art-online.org/).
// Copyright (C) 2006 Media Art Online (cafe@media-art-online.org)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// PROJECT: PGP/GnuPG Applications
// MODULE:  GnuPG support
// FILE:    GnuPGFingerprintPattern.java
//
// DESCRIPTION:
//
// REVISION HISTORY:
//  Name      Date        Description
//  ----      ----------  -----------
//  ks91      2006/10/15  initial version
//
// $Id: //depot/scm/main/java/org/media_art_online/gnupg/GnuPGFingerprintPattern.java#2 $
// $DateTime: 2007/06/26 18:15:56 $
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Package
//---------------------------------------------------------------------------
package org.media_art_online.gnupg;

//---------------------------------------------------------------------------
// Import
//---------------------------------------------------------------------------
import java.awt.*;
import java.awt.image.*;
import javax.sound.midi.*;
import javax.swing.*;

public class GnuPGFingerprintPattern extends GnuPGFingerprint {

	public GnuPGFingerprintPattern(String sHex) {
		super(sHex);
		_label = new GnuPGFingerprintLabel(this);
	}

	public BufferedImage getImage() {

		BufferedImage image = (BufferedImage)_label.createImage(
		 _label.getWidth(), _label.getHeight());
		_label.paint(image.getGraphics());

		return (image);
	}

	public Component getComponent() {
		return (_label);
	}

	public void play() {
		_label.play();
	}

//===========================================================================
// BARRIER: Anything below is not open to other files.
//===========================================================================

	private GnuPGFingerprintLabel _label;
};

class GnuPGFingerprintLabel extends JLabel {
	private static final long serialVersionUID = 1L;

	public GnuPGFingerprintLabel(GnuPGFingerprint fingerprint) {

		super();
		_fingerprint = fingerprint;
		setPreferredSize(
		 new Dimension(CNT_POLYS * PIX_HEX, CNT_POLYS * PIX_HEX));
	}

	public void paint(Graphics g) {

		String sHex = _fingerprint.getHexString();

		g.setColor(Color.lightGray);
		g.fillRect(0, 0, CNT_POLYS * PIX_HEX, CNT_POLYS * PIX_HEX);

		if (sHex == null) {
			return;
		}

		int iCol = 0;
		int iRow = 0;

		for (int i = 0; i < LEN_S_FINGERPRINT; i++) {

			g.setColor(_aoColors[hex2int(sHex.charAt(i))]);

			Point point
			 = new Point(iCol * PIX_HEX + (iRow % 2 == 0 ? PIX_HEX / 2 : 0),
			 iRow * (PIX_HEX / 2));

			g.fillRect(point.x + PIX_HEX / 4, point.y + PIX_HEX / 4,
			 PIX_HEX / 2, PIX_HEX / 2);

			if (++iCol >= (iRow % 2 == 0 ? COLS_EVEN : COLS_ODD)) {
				iCol = 0;
				iRow++;
			}
		}
	}

	public void play() {

		if (_synth == null) {

			try {
				_synth = MidiSystem.getSynthesizer();
				_synth.open();
				_channel = (_synth.getChannels())[0];

			} catch (MidiUnavailableException ex) {
				System.out.println(ex.toString());
			}
		}

		String sHex = _fingerprint.getHexString();

		for (int i = 0; i < sHex.length(); i += 4) {

			int iNote = PITCH_BASE + hex2int(sHex.charAt(i)) * PITCH_WIDTH;
			int iVelocity = VELOCITY_BASE + hex2int(sHex.charAt(i + 1))
			 * VELOCITY_WIDTH;
			int iMod = MOD_BASE + hex2int(sHex.charAt(i + 2)) * MOD_WIDTH;
			int iInterval = INTERVAL_BASE + hex2int(sHex.charAt(i + 3))
			 * INTERVAL_WIDTH;

			try {
				_channel.controlChange(CONTROL_MOD, iMod);
				_channel.noteOn(iNote, iVelocity);

				Thread.sleep(iInterval);

				_channel.allNotesOff();

			} catch (Exception unused) {
			}
		}
	}

	private int hex2int(char c) {
		return (c >= 'A' ? (c - 'A' + 0xa) : (c - '0'));
	}

	private static Color[] _aoColors = new Color[] {
		new Color(255, 255, 255),
		new Color(254,   0,   9),
		new Color(254,  26,  10),
		new Color(255, 127,  13),
		new Color(255, 179,   0),
		new Color(232, 244,   4),
		new Color(112, 189,  41),
		new Color( 35, 157,  87),
		new Color( 10, 152, 141), 
		new Color(  1, 154, 195),
		new Color( 31,  95, 169),
		new Color( 59,  28, 137),
		new Color(107,  16, 131),
		new Color(129,   1, 126),
		new Color(197,   1, 109),
		new Color(  0,   0,   0)
	};

	private GnuPGFingerprint _fingerprint;

	private static MidiChannel _channel = null;
	private static Synthesizer _synth = null;

	private static final int LEN_HEX           =   4;
	private static final int LEN_FINGERPRINT   = 160;
	private static final int LEN_S_FINGERPRINT = LEN_FINGERPRINT / LEN_HEX;

	private static final int CNT_POLYS = 5;
	private static final int COLS_EVEN = 4;
	private static final int COLS_ODD  = 5;
	private static final int PIX_HEX   = 12; /* multiple of 4 */
/*	private static final int ROWS      = 9;*/

	private static final int CONTROL_MOD = 1;

	private static final int INTERVAL_BASE  = 100;
	private static final int INTERVAL_WIDTH = 12;

	private static final int MOD_BASE = 0;
	private static final int MOD_WIDTH = 8;

	private static final int PITCH_BASE  = 24; /* C2 */
	private static final int PITCH_WIDTH =  4;

	private static final int VELOCITY_BASE = 32;
	private static final int VELOCITY_WIDTH = 6;
}

// GnuPGFingerprintPattern.java
