//---------------------------------------------------------------------------
// Freely available from Media Art Online (http://www.media-art-online.org/).
// Copyright (C) 2003, 2004 Media Art Online (cafe@media-art-online.org)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// PROJECT: PGP/GnuPG Applications
// MODULE:  GnuPG support
// FILE:    GnuPGOutput.java
//
// DESCRIPTION:
//
// REVISION HISTORY:
//  Name      Date        Description
//  ----      ----------  -----------
//  ks91      2003/02/11  initial version
//
// $Id: //depot/scm/main/java/org/media_art_online/gnupg/GnuPGOutput.java#4 $
// $DateTime: 2004/09/14 15:55:24 $
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Package
//---------------------------------------------------------------------------
package org.media_art_online.gnupg;

//---------------------------------------------------------------------------
// Import
//---------------------------------------------------------------------------
// none.

/**
 * Interface to represent GnuPG import/refresh/verficiation result.
 */
public interface GnuPGOutput {

	public static final int OUTPUT_NONE        = -1;
	public static final int OUTPUT_IMPORT      =  0;
	public static final int OUTPUT_REFRESH     =  1;
	public static final int OUTPUT_VERIFY      =  2;
	public static final int OUTPUT_FINGERPRINT =  3;

	/**
	 * Clears the past results that might have been left in the object.
	 */
	public void clear();

	/**
	 * Returns the GnuPG name of the user in concern.
	 *
	 * @return GnuPGName of the user.
	 */
	public GnuPGName getPublicKeyName();

	/**
	 * Sets the fingerprint of the key.
	 *
	 * @param sFingerprint Fingerprint.
	 */
	public void setFingerprint(String sFingerprint);

	/**
	 * Sets the output mode.
	 *
	 * @param output Output mode
	 */
	public void setOutputFor(int output);

	/**
	 * Sets the name of the user in concern.
	 *
	 * @param sName Name of the user.
	 * @param sID   ID of the public key.
	 */
	public void setPublicKeyName(String sName, String sID);

	/**
	 * Sets the validity of the public key.
	 *
	 * @param iValidity Validity of the public key.
	 */
	public void setPublicKeyValidity(int iValidity);

	/**
	 * Sets the time at which the signature is made.
	 *
	 * @param time Signature time.
	 */
	public void setSignatureTime(long time);

	/**
	 * Sets the reason of the verification result.
	 *
	 * @param reason Reason of the result.
	 */
	public void setVerifiedReason(int reason);
};

// GnuPGOutput.java
